/*
 * Copyright (C) 2001-2003 Telogis.com . All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
 * SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 */

#include <string.h>

#include <sys/heap.h>
#include <sys/event.h>
#include <netinet/ip.h>
#include <sys/socket.h>
#include <netinet/icmp.h>
#include <netinet/ip_icmp.h>


ICMPSOCKET* icmpSocketList;

ICMPSOCKET *NutIcmpCreateSocket(u_char protocol,u_short port)
{
    ICMPSOCKET *sock;

    if ((sock = NutHeapAllocClear(sizeof(ICMPSOCKET))) != 0) {
        sock->so_local_port = htons(port);
        sock->so_next = icmpSocketList;
        icmpSocketList = sock;
        sock->ttl = MAXTTL;
        sock->protocol = protocol;
    }
    return sock;
}

int NutIcmpSendTo(ICMPSOCKET *sock,u_long addr,u_char type,u_char code,void* data,u_short len)
{
    int rc;
    NETBUF *nb;

    if ((nb = NutNetBufAlloc(0, NBAF_APPLICATION, len)) == 0) {
        return -1;
    }
    memcpy(nb->nb_ap.vp, data, len);

    rc = NutIcmpReply(type,code,sock->so_local_port,htons(sock->sequence_no++),
							 addr, nb, sock->ttl);
    NutNetBufFree(nb);

    return rc;
}

int NutIcmpReceiveFrom(ICMPSOCKET *sock,u_long* addr,u_char* type,u_char* code,void* data,u_short size,u_long timeout)
{
    IPHDR *ip;
    ICMPHDR *uh;
    NETBUF *nb;

    if (sock->so_rx_nb == 0)
        NutEventWait(&sock->so_rx_rdy, timeout);

    if ((nb = sock->so_rx_nb) == 0)
        return 0;
    sock->so_rx_nb = 0;

    ip = nb->nb_nw.vp;
    *addr = ip->ip_src;

    uh = nb->nb_tp.vp;
    *type = uh->icmp_type;
    *code = uh->icmp_code;

    if (size > nb->nb_ap.sz)
        size = nb->nb_ap.sz;

    memcpy(data, nb->nb_ap.vp, size);
    NutNetBufFree(nb);

    return size;
}

int NutIcmpDestroySocket(ICMPSOCKET * sock)
{
    ICMPSOCKET *sp;
    ICMPSOCKET **spp;
    int rc = -1;

    spp = &icmpSocketList;
    sp = icmpSocketList;

    while (sp) {
        if (sp == sock) {
            *spp = sock->so_next;
            /* A packet may have arrived that the application 
               did not read before closing the socket. */
            if ( sock->so_rx_nb != 0 ) {
			NutNetBufFree(sock->so_rx_nb);
            }
            NutHeapFree(sock);
            rc = 0;
            break;
        }
        spp = &sp->so_next;
        sp = sp->so_next;
    }
    return rc;
}

ICMPSOCKET *NutIcmpFindSocket(u_short port,u_char protocol)
{
    ICMPSOCKET *sp;
    ICMPSOCKET *sock = 0;

    for (sp = icmpSocketList; sp; sp = sp->so_next) {
        if (sp->so_local_port == port && sp->protocol == protocol) {
            sock = sp;
            break;
        }
    }
    return sock;
}

int NutIcmpSetTTL(ICMPSOCKET *sock,u_char ttl){
	sock->ttl = ttl;
	return 0;
}


